searchData={"items":[{"type":"module","title":"OpenIDConnect","doc":"OpenID Connect client library for Elixir.\n\nThis library provides a complete implementation of the [OpenID Connect](https://openid.net/connect/) \nauthentication protocol, which is built on top of OAuth 2.0. It handles the complete authentication \nflow including:\n\n* Generating authorization URIs for redirecting users to identity providers\n* Fetching tokens from the token endpoint\n* Verifying JWT ID tokens using provider JWKs (JSON Web Keys)\n* Fetching user information from the userinfo endpoint\n* Supporting provider logout via end session endpoints","ref":"OpenIDConnect.html"},{"type":"module","title":"Supported Authentication Flows - OpenIDConnect","doc":"The library supports the standard OpenID Connect authentication flows:\n\n* Authorization Code Flow (most common for web applications)\n* Implicit Flow (less secure, used for JavaScript applications)\n* Hybrid Flow (combines aspects of both)","ref":"OpenIDConnect.html#module-supported-authentication-flows"},{"type":"module","title":"Supported Identity Providers - OpenIDConnect","doc":"This library works with any OpenID Connect compliant provider, including:\n\n* Google\n* Microsoft Azure AD\n* Auth0\n* Okta\n* Amazon Cognito\n* Keycloak\n* OneLogin\n* HashiCorp Vault\n* And many others","ref":"OpenIDConnect.html#module-supported-identity-providers"},{"type":"module","title":"Basic Usage - OpenIDConnect","doc":"```elixir\n# Step 1: Configure the provider\ngoogle_config = %{\n  discovery_document_uri: \"https://accounts.google.com/.well-known/openid-configuration\",\n  client_id: \"YOUR_CLIENT_ID\",\n  client_secret: \"YOUR_CLIENT_SECRET\",\n  response_type: \"code\",\n  scope: \"openid email profile\"\n}\n\n# Step 2: Generate the authorization URI (redirect the user to this URI)\n{:ok, uri} = OpenIDConnect.authorization_uri(\n  google_config,\n  \"https://example.com/auth/callback\",\n  %{state: state_token}\n)\n\n# Step 3: Exchange the authorization code for tokens\n{:ok, tokens} = OpenIDConnect.fetch_tokens(\n  google_config,\n  %{\n    code: auth_code,\n    redirect_uri: \"https://example.com/auth/callback\"\n  }\n)\n\n# Step 4: Verify the ID token\n{:ok, claims} = OpenIDConnect.verify(google_config, tokens[\"id_token\"])\n\n# Optional: Fetch additional user information\n{:ok, userinfo} = OpenIDConnect.fetch_userinfo(google_config, tokens[\"access_token\"])\n```\n\nSee the [README](readme.html) for more detailed examples and configuration options.","ref":"OpenIDConnect.html#module-basic-usage"},{"type":"function","title":"OpenIDConnect.authorization_uri/3","doc":"Builds an authorization URI for redirecting users to the identity provider.\n\nThis function creates a URL that starts the OpenID Connect authentication flow\nby redirecting the user to the provider's authorization endpoint. The endpoint URL\nis automatically retrieved from the provider's discovery document.","ref":"OpenIDConnect.html#authorization_uri/3"},{"type":"function","title":"Parameters - OpenIDConnect.authorization_uri/3","doc":"* `config` - The provider configuration map\n* `redirect_uri` - URI to redirect to after authentication (must match one registered with the provider)\n* `params` - Optional map of additional query parameters to include in the authorization URI","ref":"OpenIDConnect.html#authorization_uri/3-parameters"},{"type":"function","title":"Common Additional Parameters - OpenIDConnect.authorization_uri/3","doc":"* `state` - **Strongly recommended** - Random token to prevent CSRF attacks\n* `prompt` - Controls the login experience, common values include:\n  * `\"none\"` - No interactive prompt, fails if user authentication is required\n  * `\"login\"` - Forces the user to enter their credentials even if already logged in\n  * `\"consent\"` - Forces the consent screen to be displayed even if previously consented\n  * `\"select_account\"` - Prompts the user to select an account\n* `login_hint` - Email address or sub identifier to pre-fill the login form\n* `max_age` - Maximum elapsed time in seconds since last authentication\n* `ui_locales` - Preferred languages for the UI, space-separated list of BCP47 tags\n* `acr_values` - Authentication Context Class Reference values","ref":"OpenIDConnect.html#authorization_uri/3-common-additional-parameters"},{"type":"function","title":"Provider-Specific Parameters - OpenIDConnect.authorization_uri/3","doc":"Some providers support additional custom parameters:\n\n* Google: `hd` (hosted domain) to restrict to specific Google Workspace domains\n* Azure AD: `domain_hint` to skip the home realm discovery page\n* Many others based on the provider","ref":"OpenIDConnect.html#authorization_uri/3-provider-specific-parameters"},{"type":"function","title":"Returns - OpenIDConnect.authorization_uri/3","doc":"* `{:ok, uri}` - On success, returns the authorization URI string\n* `{:error, reason}` - On failure, returns an error tuple with details","ref":"OpenIDConnect.html#authorization_uri/3-returns"},{"type":"function","title":"Example: Basic Usage - OpenIDConnect.authorization_uri/3","doc":"```elixir\n{:ok, uri} = OpenIDConnect.authorization_uri(\n  google_config,\n  \"https://example.com/auth/callback\",\n  %{state: state_token}\n)\n```","ref":"OpenIDConnect.html#authorization_uri/3-example-basic-usage"},{"type":"function","title":"Example: With Additional Parameters - OpenIDConnect.authorization_uri/3","doc":"```elixir\n{:ok, uri} = OpenIDConnect.authorization_uri(\n  google_config,\n  \"https://example.com/auth/callback\",\n  %{\n    state: state_token,\n    prompt: \"login\",\n    login_hint: \"user@example.com\",\n    hd: \"example.com\"  # Google-specific parameter\n  }\n)\n```","ref":"OpenIDConnect.html#authorization_uri/3-example-with-additional-parameters"},{"type":"function","title":"Security: State Parameter - OpenIDConnect.authorization_uri/3","doc":"The `state` parameter is critical for preventing cross-site request forgery attacks.\nAlways include a state parameter with a secure random value and validate it when\nthe user is redirected back to your application.","ref":"OpenIDConnect.html#authorization_uri/3-security-state-parameter"},{"type":"function","title":"Creating a State Token - OpenIDConnect.authorization_uri/3","doc":"```elixir\nstate_token = Base.encode64(:crypto.strong_rand_bytes(32), padding: false)\n\n# Store the token in your session\nconn = Plug.Conn.put_session(conn, :oidc_state_token, state_token)\n\n# Include it in the authorization URI\n{:ok, uri} = OpenIDConnect.authorization_uri(\n  google_config,\n  redirect_uri,\n  %{state: state_token}\n)\n```","ref":"OpenIDConnect.html#authorization_uri/3-creating-a-state-token"},{"type":"function","title":"Validating the State Token - OpenIDConnect.authorization_uri/3","doc":"When the provider redirects back to your application, verify the state parameter:\n\n```elixir\nstored_token = Plug.Conn.get_session(conn, :oidc_state_token)\nreceived_token = params[\"state\"]\n\nif stored_token && Plug.Crypto.secure_compare(stored_token, received_token) do\n  # State token is valid, proceed with token exchange\nelse\n  # Invalid state token, reject the request\nend\n```\n\nFor more details on state tokens, see [Google's Documentation](https://developers.google.com/identity/openid-connect/openid-connect#createxsrftoken).","ref":"OpenIDConnect.html#authorization_uri/3-validating-the-state-token"},{"type":"type","title":"OpenIDConnect.client_id/0","doc":"OAuth 2.0 Client Identifier valid at the Authorization Server.","ref":"OpenIDConnect.html#t:client_id/0"},{"type":"type","title":"OpenIDConnect.client_secret/0","doc":"OAuth 2.0 Client Secret valid at the Authorization Server.","ref":"OpenIDConnect.html#t:client_secret/0"},{"type":"type","title":"OpenIDConnect.config/0","doc":"The configuration of a OpenID provider.","ref":"OpenIDConnect.html#t:config/0"},{"type":"type","title":"OpenIDConnect.discovery_document_uri/0","doc":"URL to a [OpenID Discovery Document](https://openid.net/specs/openid-connect-discovery-1_0.html) endpoint.","ref":"OpenIDConnect.html#t:discovery_document_uri/0"},{"type":"function","title":"OpenIDConnect.end_session_uri/2","doc":"Builds a URI for ending the user's session with the identity provider.\n\nThis function creates a URL for OpenID Connect RP-Initiated Logout, allowing\nyour application to sign the user out of the identity provider's session.\nThe endpoint URL is automatically retrieved from the provider's discovery document.","ref":"OpenIDConnect.html#end_session_uri/2"},{"type":"function","title":"Parameters - OpenIDConnect.end_session_uri/2","doc":"* `config` - The provider configuration map\n* `params` - Optional map of additional query parameters for the end session URI","ref":"OpenIDConnect.html#end_session_uri/2-parameters"},{"type":"function","title":"Common Additional Parameters - OpenIDConnect.end_session_uri/2","doc":"* `id_token_hint` - The ID token previously issued to the user (strongly recommended)\n* `post_logout_redirect_uri` - URI to redirect to after logout (must be registered with provider)\n* `state` - Opaque value for maintaining state between logout request and callback","ref":"OpenIDConnect.html#end_session_uri/2-common-additional-parameters"},{"type":"function","title":"Provider Requirements - OpenIDConnect.end_session_uri/2","doc":"Different providers have different requirements for logout:\n\n* Azure AD: Requires `post_logout_redirect_uri` and may accept `id_token_hint`\n* Auth0: Requires `client_id` and `returnTo` (instead of `post_logout_redirect_uri`)\n* Cognito: Requires `client_id` and `logout_uri` (their name for redirect URI)\n* Okta: Requires `id_token_hint` and accepts `post_logout_redirect_uri`\n* Keycloak: Accepts `id_token_hint` and `post_logout_redirect_uri`\n\nAlways consult your provider's documentation for specific requirements.","ref":"OpenIDConnect.html#end_session_uri/2-provider-requirements"},{"type":"function","title":"Returns - OpenIDConnect.end_session_uri/2","doc":"* `{:ok, uri}` - On success, returns the end session URI string\n* `{:error, :endpoint_not_set}` - If the provider doesn't support RP-Initiated Logout\n* `{:error, reason}` - On other failures, returns an error tuple with details","ref":"OpenIDConnect.html#end_session_uri/2-returns"},{"type":"function","title":"Provider Support Note - OpenIDConnect.end_session_uri/2","doc":"Not all providers support RP-Initiated Logout. If the provider's discovery document\ndoesn't include an `end_session_endpoint`, this function will return\n`{:error, :endpoint_not_set}`. In such cases, you'll need to implement session\ntermination in your application only.","ref":"OpenIDConnect.html#end_session_uri/2-provider-support-note"},{"type":"function","title":"Example: Azure AD Logout - OpenIDConnect.end_session_uri/2","doc":"```elixir\n{:ok, logout_uri} = OpenIDConnect.end_session_uri(\n  azure_config,\n  %{\n    post_logout_redirect_uri: \"https://example.com/logged-out\"\n  }\n)\n\n# Redirect the user to the logout_uri\nredirect(conn, external: logout_uri)\n```","ref":"OpenIDConnect.html#end_session_uri/2-example-azure-ad-logout"},{"type":"function","title":"Example: With ID Token Hint - OpenIDConnect.end_session_uri/2","doc":"```elixir\n{:ok, logout_uri} = OpenIDConnect.end_session_uri(\n  okta_config,\n  %{\n    id_token_hint: id_token,\n    post_logout_redirect_uri: \"https://example.com/logged-out\"\n  }\n)\n```\n\nFor more details, see the [OpenID Connect RP-Initiated Logout Specification](https://openid.net/specs/openid-connect-rpinitiated-1_0.html).","ref":"OpenIDConnect.html#end_session_uri/2-example-with-id-token-hint"},{"type":"function","title":"OpenIDConnect.fetch_tokens/2","doc":"Fetches authentication tokens from the provider using the token endpoint.\n\nThis function exchanges the authorization code or refresh token for access and ID tokens\nfrom the identity provider's token endpoint. The endpoint URL is automatically retrieved\nfrom the provider's discovery document.","ref":"OpenIDConnect.html#fetch_tokens/2"},{"type":"function","title":"Parameters - OpenIDConnect.fetch_tokens/2","doc":"* `config` - The provider configuration map\n* `params` - A map of parameters to send to the token endpoint","ref":"OpenIDConnect.html#fetch_tokens/2-parameters"},{"type":"function","title":"Common Parameters - OpenIDConnect.fetch_tokens/2","doc":"The params map should include different fields depending on the grant type:","ref":"OpenIDConnect.html#fetch_tokens/2-common-parameters"},{"type":"function","title":"Authorization Code Grant (most common) - OpenIDConnect.fetch_tokens/2","doc":"```elixir\n%{\n  code: \"AUTHORIZATION_CODE_FROM_CALLBACK\",\n  redirect_uri: \"https://example.com/callback\", # Must match the original redirect_uri\n  grant_type: \"authorization_code\" # Optional, defaults to \"authorization_code\"\n}\n```","ref":"OpenIDConnect.html#fetch_tokens/2-authorization-code-grant-most-common"},{"type":"function","title":"Refresh Token Grant - OpenIDConnect.fetch_tokens/2","doc":"```elixir\n%{\n  refresh_token: \"REFRESH_TOKEN\",\n  grant_type: \"refresh_token\"\n}\n```","ref":"OpenIDConnect.html#fetch_tokens/2-refresh-token-grant"},{"type":"function","title":"Returns - OpenIDConnect.fetch_tokens/2","doc":"* `{:ok, tokens}` - On successful token exchange, returns a map containing at minimum:\n  * `\"access_token\"` - The OAuth 2.0 access token\n  * `\"token_type\"` - The token type (typically \"Bearer\")\n  * `\"expires_in\"` - Token lifetime in seconds\n  * `\"id_token\"` - The OpenID Connect ID token (JWT)\n  * `\"refresh_token\"` - Optional refresh token for obtaining new access tokens\n\n* `{:error, reason}` - On failure, returns an error tuple with reason","ref":"OpenIDConnect.html#fetch_tokens/2-returns"},{"type":"function","title":"Example - OpenIDConnect.fetch_tokens/2","doc":"```elixir\n{:ok, tokens} = OpenIDConnect.fetch_tokens(\n  google_config,\n  %{\n    code: params[\"code\"],\n    redirect_uri: \"https://example.com/auth/callback\" \n  }\n)\n\n# Access the tokens\naccess_token = tokens[\"access_token\"]\nid_token = tokens[\"id_token\"]\nrefresh_token = tokens[\"refresh_token\"]\n```\n\nFor more details, see the [OpenID Connect spec](https://openid.net/specs/openid-connect-core-1_0.html#TokenEndpoint).","ref":"OpenIDConnect.html#fetch_tokens/2-example"},{"type":"function","title":"OpenIDConnect.fetch_userinfo/2","doc":"Fetches information about the authenticated user from the userinfo endpoint.\n\nThis function retrieves additional claims about the end-user from the provider's\nuserinfo endpoint using the access token. This can be useful when you need user\nattributes that weren't included in the ID token claims.","ref":"OpenIDConnect.html#fetch_userinfo/2"},{"type":"function","title":"Parameters - OpenIDConnect.fetch_userinfo/2","doc":"* `config` - The provider configuration map\n* `access_token` - The OAuth 2.0 access token obtained from `fetch_tokens/2`","ref":"OpenIDConnect.html#fetch_userinfo/2-parameters"},{"type":"function","title":"Returns - OpenIDConnect.fetch_userinfo/2","doc":"* `{:ok, userinfo}` - On success, returns a map of user information\n* `{:error, reason}` - On failure, returns an error tuple with details","ref":"OpenIDConnect.html#fetch_userinfo/2-returns"},{"type":"function","title":"Userinfo Claims - OpenIDConnect.fetch_userinfo/2","doc":"The returned claims will depend on the scopes requested during authorization,\nbut typically include:\n\n* `\"sub\"` - Subject identifier (unique user ID)\n* `\"name\"` - User's full name (if \"profile\" scope)\n* `\"given_name\"` - First name (if \"profile\" scope)\n* `\"family_name\"` - Last name (if \"profile\" scope)\n* `\"email\"` - Email address (if \"email\" scope)\n* `\"email_verified\"` - Boolean indicating if email is verified (if \"email\" scope)\n* `\"picture\"` - URL to profile picture (if \"profile\" scope)","ref":"OpenIDConnect.html#fetch_userinfo/2-userinfo-claims"},{"type":"function","title":"Example - OpenIDConnect.fetch_userinfo/2","doc":"```elixir\n{:ok, userinfo} = OpenIDConnect.fetch_userinfo(google_config, access_token)\n\n# Access user information\nuser_id = userinfo[\"sub\"]\nemail = userinfo[\"email\"]\nname = userinfo[\"name\"]\n```","ref":"OpenIDConnect.html#fetch_userinfo/2-example"},{"type":"function","title":"Security Note - OpenIDConnect.fetch_userinfo/2","doc":"The userinfo endpoint requires a valid access token. The token must have\nappropriate scopes (typically \"openid\" and others like \"profile\" or \"email\").\n\nFor more details, see the [OpenID Connect UserInfo Endpoint](https://openid.net/specs/openid-connect-core-1_0.html#UserInfo).","ref":"OpenIDConnect.html#fetch_userinfo/2-security-note"},{"type":"type","title":"OpenIDConnect.jwt/0","doc":"JSON Web Token\n\nSee: https://jwt.io/introduction/","ref":"OpenIDConnect.html#t:jwt/0"},{"type":"type","title":"OpenIDConnect.redirect_uri/0","doc":"Redirection URI to which the response will be sent.\n\nThis URI MUST exactly match one of the Redirection URI values for the Client pre-registered at the OpenID Provider,\nwith the matching performed as described in Section 6.2.1 of [RFC3986] (Simple String Comparison).\n\nWhen using this flow, the Redirection URI SHOULD use the https scheme; however, it MAY use the http scheme,\nprovided that the Client Type is confidential, as defined in Section 2.1 of OAuth 2.0,\nand provided the OP allows the use of http Redirection URIs in this case. The Redirection URI MAY use an alternate scheme,\nsuch as one that is intended to identify a callback into a native application.","ref":"OpenIDConnect.html#t:redirect_uri/0"},{"type":"type","title":"OpenIDConnect.response_type/0","doc":"OAuth 2.0 Response Type value that determines the authorization processing flow to be used,\nincluding what parameters are returned from the endpoints used.","ref":"OpenIDConnect.html#t:response_type/0"},{"type":"type","title":"OpenIDConnect.scope/0","doc":"OAuth 2.0 Scope Values that the Client is declaring that it will restrict itself to using.","ref":"OpenIDConnect.html#t:scope/0"},{"type":"function","title":"OpenIDConnect.verify/2","doc":"Verifies the validity and authenticity of an OpenID Connect ID token.\n\nThis security-critical function verifies that:\n\n1. The token's signature is valid and was signed by the provider's key\n2. The token has not expired (checking the \"exp\" claim)\n3. The token is intended for your application (checking the \"aud\" claim)","ref":"OpenIDConnect.html#verify/2"},{"type":"function","title":"Parameters - OpenIDConnect.verify/2","doc":"* `config` - The provider configuration map\n* `jwt` - The ID token string (a JSON Web Token) from the tokens response","ref":"OpenIDConnect.html#verify/2-parameters"},{"type":"function","title":"Returns - OpenIDConnect.verify/2","doc":"* `{:ok, claims}` - On successful verification, returns the decoded claims from the token\n* `{:error, reason}` - On verification failure, returns an error tuple with details","ref":"OpenIDConnect.html#verify/2-returns"},{"type":"function","title":"Verification Process - OpenIDConnect.verify/2","doc":"1. The signature is verified using the provider's JSON Web Keys (JWKs), which are fetched\n   from the provider's JWKs endpoint listed in the discovery document.\n2. The \"exp\" (expiration) claim is checked to ensure the token has not expired.\n   A configurable leeway (default: 30 seconds) is allowed to account for clock skew.\n3. The \"aud\" (audience) claim is verified to ensure the token is intended for your\n   application, as identified by your client_id.","ref":"OpenIDConnect.html#verify/2-verification-process"},{"type":"function","title":"Example - OpenIDConnect.verify/2","doc":"```elixir\n{:ok, claims} = OpenIDConnect.verify(google_config, id_token)\n\n# Common claims you might find in the response:\nuser_id = claims[\"sub\"]         # Unique user identifier\nemail = claims[\"email\"]         # User's email (if in scope)\nname = claims[\"name\"]           # User's name (if in scope)\npicture = claims[\"picture\"]     # URL to user's profile picture (if available)\nissuer = claims[\"iss\"]          # Identifies the token issuer\n```","ref":"OpenIDConnect.html#verify/2-example"},{"type":"function","title":"Security Warning - OpenIDConnect.verify/2","doc":"Always verify tokens before trusting their contents. Never use token data for\nauthentication purposes without verification, as tokens could be forged or tampered with.","ref":"OpenIDConnect.html#verify/2-security-warning"},{"type":"module","title":"OpenIDConnect.Document","doc":"","ref":"OpenIDConnect.Document.html"},{"type":"function","title":"OpenIDConnect.Document.__struct__/0","doc":"This module caches OIDC documents and their JWKs for a limited timeframe, which is min(`@refresh_time`, `document.remaining_lifetime`).","ref":"OpenIDConnect.Document.html#__struct__/0"},{"type":"function","title":"OpenIDConnect.Document.fetch_document/1","doc":"Fetches an OIDC document from cache, falling back to fetching from the remote resource","ref":"OpenIDConnect.Document.html#fetch_document/1"},{"type":"type","title":"OpenIDConnect.Document.t/0","doc":"","ref":"OpenIDConnect.Document.html#t:t/0"},{"type":"extras","title":"OpenIDConnect","doc":"# OpenIDConnect\n\nClient library for consuming and working with OpenID Connect Providers\n\n**[OpenIDConnect is built and maintained by DockYard, contact us for expert Elixir and Phoenix consulting](https://dockyard.com/phoenix-consulting)**.","ref":"readme.html"},{"type":"extras","title":"Installation - OpenIDConnect","doc":"[Available in Hex](https://hex.pm/packages/openid_connect), the package can be installed as:\n\nAdd `openid_connect` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:openid_connect, \"~> 1.0.0\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - OpenIDConnect","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Configuration - OpenIDConnect","doc":"This library works with any OpenID Connect provider through a standard configuration map. You'll need to create a configuration map for each provider you want to work with:\n\n```elixir\ngoogle_config = %{\n  discovery_document_uri: \"https://accounts.google.com/.well-known/openid-configuration\",\n  client_id: \"CLIENT_ID\",\n  client_secret: \"CLIENT_SECRET\",\n  redirect_uri: \"https://example.com/session\",\n  response_type: \"code\",\n  scope: \"openid email profile\"\n}\n```\n\nThe configuration requires:\n- `discovery_document_uri`: URL to the OpenID Connect Discovery Document\n- `client_id`: OAuth 2.0 Client Identifier provided by your identity provider\n- `client_secret`: OAuth 2.0 Client Secret provided by your identity provider\n- `response_type`: OAuth 2.0 Response Type (typically \"code\" for authorization code flow)\n- `scope`: Space-separated list of scopes (must include \"openid\")\n\nAdditional optional configuration:\n- `leeway`: Number of seconds to allow for clock skew when verifying token expiration (default: 30)\n\nMost major OAuth2 providers support OpenID Connect, including:\n- Google\n- Microsoft Azure AD\n- Auth0\n- Okta\n- AWS Cognito\n- Keycloak\n- OneLogin\n- Many others\n\nFor a full list, see [major OpenID Connect providers](https://en.wikipedia.org/wiki/List_of_OAuth_providers).","ref":"readme.html#configuration"},{"type":"extras","title":"Usage - OpenIDConnect","doc":"Implementing OpenID Connect authentication in your app involves these steps:\n\n1. Generate an authorization URI and redirect the user to it\n2. Handle the callback from the provider after successful authentication\n3. Exchange the authorization code for tokens\n4. Verify the ID token\n5. Use the claims to establish a user session\n\n#### 1. Generate the Authorization URI\n\nCreate a URL that will redirect the user to the provider's login page:\n\n```elixir\n{:ok, uri} = OpenIDConnect.authorization_uri(\n  google_config,\n  \"https://example.com/auth/callback\",\n  %{state: state_token}\n)\n```\n\nYou should:\n- Generate a secure state token for CSRF protection\n- Store this state token in your session\n- Include the state token in the authorization URI\n\n#### 2. Handle the Callback from the Provider\n\nAfter the user logs in, the provider redirects back to your `redirect_uri` with an authorization code and the state token you provided:\n\n```elixir\n# In your callback controller/handler\ndef callback(conn, params) do\n  # First verify that the state parameter matches the one we stored\n  stored_state = get_session(conn, :state_token)\n  received_state = params[\"state\"]\n  \n  if Plug.Crypto.secure_compare(stored_state, received_state) do\n    # State is valid, proceed with code exchange\n    handle_valid_callback(conn, params)\n  else\n    # State token doesn't match, reject the request\n    conn\n    |> put_status(401)\n    |> render(\"error.html\", error: \"Invalid state parameter\")\n  end\nend\n```\n\n#### 3. Exchange the Authorization Code for Tokens\n\nExchange the code for ID and access tokens:\n\n```elixir\n{:ok, tokens} = OpenIDConnect.fetch_tokens(google_config, %{\n  code: params[\"code\"],\n  redirect_uri: \"https://example.com/auth/callback\"  # Must match original redirect_uri\n})\n\n# tokens will contain:\n# %{\n#   \"access_token\" => \"ya29.a0AfB_...\",\n#   \"id_token\" => \"eyJhbGciOiJSUzI1...\",\n#   \"expires_in\" => 3599,\n#   \"token_type\" => \"Bearer\",\n#   \"refresh_token\" => \"1//03s...\" (if requested)\n# }\n```\n\n#### 4. Verify the ID Token\n\nThe ID token is a JWT containing claims about the user. Always verify it before trusting its contents:\n\n```elixir\n{:ok, claims} = OpenIDConnect.verify(google_config, tokens[\"id_token\"])\n\n# claims will contain user information like:\n# %{\n#   \"sub\" => \"10865933565....\",  # Unique user identifier\n#   \"email\" => \"user@example.com\",\n#   \"email_verified\" => true,\n#   \"name\" => \"User Name\",\n#   \"picture\" => \"https://...\",\n#   \"given_name\" => \"User\",\n#   \"family_name\" => \"Name\",\n#   \"locale\" => \"en\",\n#   \"iat\" => 1585662674,  # Issued at timestamp\n#   \"exp\" => 1585666274,  # Expiration timestamp\n#   \"aud\" => \"YOUR_CLIENT_ID\",\n#   \"iss\" => \"https://accounts.google.com\"\n# }\n```\n\n#### 5. Use the Claims to Establish a User Session\n\nUse the verified claims to identify your user and establish a session:\n\n```elixir\n# The \"sub\" claim is the unique identifier for the user\nuser_id = claims[\"sub\"]\n\n# Find or create a user in your system\nuser = find_or_create_user(user_id, claims)\n\n# Establish a session for this user\nconn\n|> put_session(:user_id, user.id)\n|> redirect(to: \"/dashboard\")\n```\n\n#### Optional: Fetch Additional User Information\n\nSome providers allow you to fetch additional user information using the access token:\n\n```elixir\n{:ok, userinfo} = OpenIDConnect.fetch_userinfo(google_config, tokens[\"access_token\"])\n```\n\n#### Ending a User Session\n\nSome providers support log out via an end session endpoint:\n\n```elixir\n{:ok, logout_uri} = OpenIDConnect.end_session_uri(\n  google_config,\n  %{id_token_hint: id_token, post_logout_redirect_uri: \"https://example.com/logout/callback\"}\n)\n\n# Redirect the user to the logout_uri\nredirect(conn, external: logout_uri)\n```\n\n**Key Security Considerations:**\n\n1. Always validate the state token\n2. Store secrets (client ID, client secret) in environment variables\n3. Verify the ID token before trusting its contents\n4. Use HTTPS for all redirect URIs\n5. Consider token storage carefully - sessions are usually appropriate\n6. Clear tokens when logging out","ref":"readme.html#usage"},{"type":"extras","title":"Authors - OpenIDConnect","doc":"- [Brian Cardarella](http://twitter.com/bcardarella)\n\n[We are very thankful for the many contributors](https://github.com/dockyard/openid_connect/graphs/contributors)","ref":"readme.html#authors"},{"type":"extras","title":"Versioning - OpenIDConnect","doc":"This library follows [Semantic Versioning](http://semver.org)","ref":"readme.html#versioning"},{"type":"extras","title":"Looking for help with your Elixir project? - OpenIDConnect","doc":"[At DockYard we are ready to help you build your next Elixir project](https://dockyard.com/phoenix-consulting). We have a unique expertise in Elixir and Phoenix development that is unmatched. [Get in touch!](https://dockyard.com/contact/hire-us)\n\nAt DockYard we love Elixir! You can [read our Elixir blog posts](https://dockyard.com/blog/categories/elixir)\nor come visit us at [The Boston Elixir Meetup](http://www.meetup.com/Boston-Elixir/) that we organize.","ref":"readme.html#looking-for-help-with-your-elixir-project"},{"type":"extras","title":"Want to help? - OpenIDConnect","doc":"Please do! We are always looking to improve this library. Please see our\n[Contribution Guidelines](https://github.com/dockyard/openid_connect/blob/master/CONTRIBUTING.md)\non how to properly submit issues and pull requests.","ref":"readme.html#want-to-help"},{"type":"extras","title":"Sponsors - OpenIDConnect","doc":"In addition to our contributors, this library receives support and maintenance from [TV Labs](https://tvlabs.ai)","ref":"readme.html#sponsors"},{"type":"extras","title":"Legal - OpenIDConnect","doc":"[DockYard](http://dockyard.com/), Inc. &copy; 2018\n\n[@dockyard](http://twitter.com/dockyard)\n\n[Licensed under the MIT license](http://www.opensource.org/licenses/mit-license.php)","ref":"readme.html#legal"},{"type":"extras","title":"Complete Phoenix Example","doc":"# Complete Phoenix Example\n\nHere's a complete example of implementing OpenID Connect authentication in a Phoenix application:\n\n```elixir\n# Configuration in config/config.exs\nconfig :my_app, :google_oidc_config, %{\n  discovery_document_uri: \"https://accounts.google.com/.well-known/openid-configuration\",\n  client_id: System.get_env(\"GOOGLE_CLIENT_ID\"),\n  client_secret: System.get_env(\"GOOGLE_CLIENT_SECRET\"),\n  response_type: \"code\",\n  scope: \"openid email profile\"\n}\n\n# Routes in lib/my_app_web/router.ex\nscope \"/auth\", MyAppWeb do\n  pipe_through [:browser, :redirect_if_authenticated]\n  \n  get \"/google\", AuthController, :request\n  get \"/google/callback\", AuthController, :callback\n  get \"/logout\", AuthController, :logout\nend\n\n# Controller in lib/my_app_web/controllers/auth_controller.ex\ndefmodule MyAppWeb.AuthController do\n  use MyAppWeb, :controller\n  \n  @doc \"\"\"\n  Initiates the OpenID Connect authentication flow\n  \"\"\"\n  def request(conn, _params) do\n    google_config = Application.fetch_env!(:my_app, :google_oidc_config)\n    \n    # Generate a secure state token to prevent CSRF\n    state_token = Base.encode64(:crypto.strong_rand_bytes(32), padding: false)\n    \n    # The redirect_uri must match what your provider has configured\n    redirect_uri = Routes.auth_url(conn, :callback)\n    \n    {:ok, auth_uri} = OpenIDConnect.authorization_uri(\n      google_config,\n      redirect_uri,\n      %{state: state_token}\n    )\n    \n    conn\n    |> put_session(:oidc_state_token, state_token)\n    |> redirect(external: auth_uri)\n  end\n  \n  @doc \"\"\"\n  Handles the callback from the OpenID provider after successful authentication\n  \"\"\"\n  def callback(conn, params) do\n    google_config = Application.fetch_env!(:my_app, :google_oidc_config)\n    \n    # Verify the state parameter to prevent CSRF attacks\n    with true <- valid_state_token?(conn, params[\"state\"]),\n         {:ok, tokens} <- OpenIDConnect.fetch_tokens(\n           google_config,\n           %{\n             code: params[\"code\"],\n             redirect_uri: Routes.auth_url(conn, :callback)\n           }\n         ),\n         {:ok, claims} <- OpenIDConnect.verify(google_config, tokens[\"id_token\"]) do\n      \n      # Store tokens securely, typically in the session or a secure cookie\n      # Storing in the session should be fine for most cases\n      conn = put_session(conn, :access_token, tokens[\"access_token\"])\n      \n      # Find or create the user based on the claims\n      user = \n        case Accounts.get_user_by_provider_id(\"google\", claims[\"sub\"]) do\n          nil -> \n            {:ok, user} = Accounts.create_user(%{\n              email: claims[\"email\"],\n              name: claims[\"name\"],\n              provider: \"google\",\n              provider_id: claims[\"sub\"]\n            })\n            user\n          user -> \n            user\n        end\n      \n      # Establish the user's session\n      conn\n      |> put_session(:current_user_id, user.id)\n      |> configure_session(renew: true)\n      |> put_flash(:info, \"Successfully signed in!\")\n      |> redirect(to: Routes.dashboard_path(conn, :index))\n    else\n      false -> \n        # Invalid state token, potential CSRF attack\n        conn\n        |> put_flash(:error, \"Authentication failed: Invalid state parameter\")\n        |> redirect(to: Routes.page_path(conn, :index))\n      \n      {:error, reason} ->\n        # Log the error for debugging\n        require Logger\n        Logger.error(\"Authentication error: #{inspect(reason)}\")\n        \n        conn\n        |> put_flash(:error, \"Authentication failed\")\n        |> redirect(to: Routes.page_path(conn, :index))\n    end\n  end\n  \n  @doc \"\"\"\n  Logs the user out by clearing the session\n  \"\"\"\n  def logout(conn, _params) do\n    google_config = Application.fetch_env!(:my_app, :google_oidc_config)\n    id_token = get_session(conn, :id_token)\n    \n    # Clear our local session\n    conn = \n      conn\n      |> configure_session(drop: true)\n      |> put_flash(:info, \"Successfully signed out!\")\n    \n    # If we have an ID token, try to use the provider's logout endpoint\n    if id_token do\n      case OpenIDConnect.end_session_uri(\n        google_config,\n        %{\n          id_token_hint: id_token,\n          post_logout_redirect_uri: Routes.page_url(conn, :index)\n        }\n      ) do\n        {:ok, logout_uri} ->\n          redirect(conn, external: logout_uri)\n        _ ->\n          # Fall back to local logout if provider doesn't support end_session\n          redirect(conn, to: Routes.page_path(conn, :index))\n      end\n    else\n      redirect(conn, to: Routes.page_path(conn, :index))\n    end\n  end\n  \n  # Verify the state token to prevent CSRF attacks\n  defp valid_state_token?(conn, token) do\n    stored_token = get_session(conn, :oidc_state_token)\n    \n    # Clear the token to prevent replay attacks\n    conn = delete_session(conn, :oidc_state_token)\n    \n    case stored_token do\n      nil -> false\n      ^token -> true\n      _ -> false\n    end\n  end\nend\n```","ref":"phoenix_example.html"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"v1.0.0 - CHANGELOG","doc":"Complete rewrite of the library by @AndrewDryga","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"Added - CHANGELOG","doc":"* Add `end_session_uri/2` and `fetch_userinfo/2`\n* Adds OpenID claim validation","ref":"changelog.html#added"},{"type":"extras","title":"Removed - CHANGELOG","doc":"* GenServer bottleneck\n* Atom requirement for provider name\n* Application config from the library","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - CHANGELOG","doc":"* Rewrote tests to better cover production code\n* Use Finch/Mint as the HTTP client instead of HTTPoison\n\nPlease see the documentation for migrating from prior versions.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.2.2 - CHANGELOG","doc":"* Allow missing `claims_supported` in discovery document\n* Allow overriding document params","ref":"changelog.html#v0-2-2"},{"type":"extras","title":"v0.2.1 - CHANGELOG","doc":"* Relaxed jason version requirement","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"v0.2.0 - CHANGELOG","doc":"* BREAKING CHANGE - Multiple response types supported. See: PR #17 for details","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.2 - CHANGELOG","doc":"* Optional params for `authorization_uri`","ref":"changelog.html#v0-1-2"},{"type":"extras","title":"v0.1.1 - CHANGELOG","doc":"* Remove Poison (legacy dep)\n* Force JOSE to use Jason in test suite","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"v0.1.0 - CHANGELOG","doc":"* Initial public release","ref":"changelog.html#v0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}